//! Dynamic data storage and transfer types for use in [`io`]

/// Owned data type, for use mostly in outputs and storage
#[derive(Clone, Debug)]
pub enum Data {
    String(String),
    Int(i32),
}

impl From<String> for Data {
    fn from(value: String) -> Self {
        Self::String(value)
    }
}

impl From<i32> for Data {
    fn from(value: i32) -> Self {
        Self::Int(value)
    }
}

/// Unowned data type, for inputs into runner functions
#[derive(Clone, Copy, Debug)]
pub enum DataRef<'a> {
    String(&'a str),
    Int(i32),
}

impl DataRef<'_> {
    /// converts itself to `OwnedData`
    pub fn to_owned_data(&self) -> Data {
        match self {
            DataRef::String(s) => (*s).to_owned().into(),
            DataRef::Int(i) => (*i).into(),
        }
    }
}

impl<'a> From<&'a str> for DataRef<'a> {
    fn from(value: &'a str) -> Self {
        Self::String(value)
    }
}

impl From<i32> for DataRef<'_> {
    fn from(value: i32) -> Self {
        Self::Int(value)
    }
}

impl<'a> From<&'a Data> for DataRef<'a> {
    fn from(value: &'a Data) -> Self {
        match value {
            Data::String(s) => DataRef::String(s),
            Data::Int(i) => DataRef::Int(*i),
        }
    }
}
